# __version__ is used in docker run command to identify
# the version of docker image to run
__version__ = "0.0.39"

supported_virt_types = ["pipenv", "conda"]

app_config_file = "atri.app.json"

assets_dir = "assets"
atri_templates_dir = "atri_templates"
controllers_dir = "controllers"
atri_app_dir = "atri_app"